#!/usr/bin/env python3

import argparse
import os
import os.path
import re
import sys
import urllib.request
import hashlib
import shutil
import json
import tarfile
import tempfile
import zipfile
from pep508_parser import parser
from pipoe import licenses
from functools import partial
from collections import namedtuple
from pprint import pformat

import pkginfo

BB_TEMPLATE = """
SUMMARY = "{summary}"
HOMEPAGE = "{homepage}"
AUTHOR = "{author} <{author_email}>"
LICENSE = "{license}"
LIC_FILES_CHKSUM = "file://{license_file};md5={license_md5}"

SRC_URI = "{src_uri}"
SRC_URI[md5sum] = "{md5}"
SRC_URI[sha256sum] = "{sha256}"

S = "${{WORKDIR}}/{src_dir}"

RDEPENDS_${{PN}} = "{dependencies}"

inherit setuptools{setuptools}
"""

BB_EXTRA_TEMPLATE = """
SUMMARY = "{summary}"
HOMEPAGE = "{homepage}"
AUTHOR = "{author} <{author_email}>"

RDEPENDS_${{PN}} = "{dependencies}"

inherit packagegroup
"""


Package = namedtuple(
    "Package",
    [
        "name",
        "version",
        "summary",
        "homepage",
        "author",
        "author_email",
        "license",
        "license_file",
        "license_md5",
        "src_dir",
        "src_uri",
        "src_md5",
        "src_sha256",
        "dependencies",
    ],
)

Dependency = namedtuple("Dependency", ["name", "version", "extra"])


def md5sum(path):
    with open(path, mode="rb") as f:
        d = hashlib.md5()
        for buf in iter(partial(f.read, 128), b""):
            d.update(buf)
    return d.hexdigest()


def sha256sum(path):
    with open(path, mode="rb") as f:
        d = hashlib.sha256()
        for buf in iter(partial(f.read, 128), b""):
            d.update(buf)
    return d.hexdigest()


def package_to_bb_name(package):
    return package.lower().replace("_", "-").replace(".", "-")


def translate_license(license, default_license):
    try:
        return licenses.LICENSES[license]
    except:
        if default_license:
            return default_license

        print("Failed to translate license: {}".format(license))
        mapping = input("Please enter a valid license name: ")
        licenses.LICENSES[license] = mapping
        return mapping


def unpack_package(file):
    tmpdir = "{}.d".format(file)

    if os.path.exists(tmpdir):
        shutil.rmtree(tmpdir)

    os.mkdir(tmpdir)
    shutil.unpack_archive(file, extract_dir=tmpdir)

    return tmpdir


def get_file_extension(uri):
    extensions = ["tar", "tar.gz", "tar.bz2", "tar.xz", "zip"]
    for extension in extensions:
        if uri.endswith(extension):
            return extension
    raise Exception("Extension not supported: {}".format(uri))


def get_package_file_info(package, version, uri):
    extension = get_file_extension(uri)
    output = "/tmp/{}_{}.{}".format(package, version, extension)

    if os.path.exists(output):
        os.remove(output)

    urllib.request.urlretrieve(uri, output)

    tmpdir = unpack_package(output)
    src_dir = os.listdir(tmpdir)[0]

    src_files = os.listdir("{}/{}".format(tmpdir, src_dir))

    try:
        license_file = next(
            f
            for f in src_files
            if ("license" in f.lower() or "copying" in f.lower())
            and not os.path.isdir(os.path.join(tmpdir, src_dir, f))
        )
    except:
        license_file = "setup.py"

    license_path = os.path.join(tmpdir, src_dir, license_file)
    license_md5 = md5sum(license_path)
    src_md5 = md5sum(output)
    src_sha256 = sha256sum(output)

    os.remove(output)
    shutil.rmtree(tmpdir)

    return (src_md5, src_sha256, src_dir, license_file, license_md5)


def decide_version(spec):
    version = spec[2]
    if version:
        version = version[0]
        relation = version[0]
        version = version[1]

        if relation == "==":
            return version
        elif relation == ">=":
            return None
        elif relation == "<=":
            return version
        else:
            return None
    else:
        return None


def decide_extra(spec):
    extra = spec[3]
    if extra:
        if extra[0] == "and":
            return extra[2][2]
        else:
            return extra[2]
    else:
        return None


def parse_requires_dist(requires_dist):
    spec = parser.parse(requires_dist)
    ret = Dependency(spec[0], decide_version(spec), decide_extra(spec))
    return ret


def fetch_requirements_from_remote_package(info, version):
    """ Looks up requires_dist from an actual package """

    # Version must exists, otherwise previous steps failed
    pkg_versions = info["releases"][version]

    # If we must fetch a package, lets fetch the smallest one
    pkg_url = sorted(pkg_versions, key=lambda pkg: pkg["size"], reverse=True)[0]["url"]
    filename = pkg_url.split("/")[-1]

    # Select the appropriate parser from pkginfo based on the filename
    parse = None
    if filename.endswith(".tar.gz"):
        parse = pkginfo.SDist
    elif filename.endswith(".whl"):
        parse = pkginfo.Wheel
    elif filename.endswith(".egg"):
        parse =pkginfo.BDist
    else:
        raise RuntimeError("Unsupported fileformat for package introspection: {}".format(filename))

    # Download the package and read the MANIFEST
    with tempfile.TemporaryDirectory() as directory:
        path = os.path.join(directory, filename)
        urllib.request.urlretrieve(pkg_url, path)
        return parse(path).requires_dist


def get_package_dependencies(requires_dist, follow_extras=False):
    deps = []

    if requires_dist:
        for dep in requires_dist:
            d = parse_requires_dist(dep)
            if d.extra and not follow_extras:
                continue
            deps.append(d)

    return deps


PROCESSED_PACKAGES = []


def get_package_info(
    package,
    version=None,
    packages=None,
    indent=0,
    extra=None,
    follow_extras=False,
    default_license=None,
):
    global PROCESSED_PACKAGES

    if not packages:
        packages = [[]]
    elif package in [package.name for package in PROCESSED_PACKAGES] or package in [
        package.name for package in packages[0]
    ]:
        return packages[0]

    indent_str = ""
    if indent:
        indent_str = "|" + (indent - 2) * "-" + " "

    extra_str = ""
    if extra:
        extra_str = "[{}]".format(extra)

    print(
        "  {}{}{}{}".format(
            indent_str, package, extra_str, "=={}".format(version) if version else ""
        )
    )

    try:
        if version:
            url = "https://pypi.org/pypi/{}/{}/json".format(package, version)
        else:
            url = "https://pypi.org/pypi/{}/json".format(package)

        response = urllib.request.urlopen(url).read().decode(encoding="UTF-8")
        info = json.loads(response)

        name = package
        version = info["info"]["version"]
        summary = info["info"]["summary"]
        homepage = info["info"]["home_page"]
        author = info["info"]["author"]
        author_email = info["info"]["author_email"]
        license = translate_license(info["info"]["license"], default_license)

        try:
            version_info = next(
                i for i in info["releases"][version] if i["packagetype"] == "sdist"
            )
        except:
            raise Exception("No sdist package can be found.")

        src_uri = version_info["url"]
        src_md5, src_sha256, src_dir, license_file, license_md5 = get_package_file_info(
            package, version, src_uri
        )

        requires_dist = info["info"]["requires_dist"]

        # Only parse if requires_dist is missing, e.g. sentry-sdk
        if requires_dist is None:
            requires_dist = fetch_requirements_from_remote_package(info, version)

        dependencies = get_package_dependencies(requires_dist, follow_extras=follow_extras)

        package = Package(
            name,
            version,
            summary,
            homepage,
            author,
            author_email,
            license,
            license_file,
            license_md5,
            src_dir,
            src_uri,
            src_md5,
            src_sha256,
            dependencies,
        )

        packages[0].append(package)
        PROCESSED_PACKAGES.append(package)

        for dependency in dependencies:
            get_package_info(
                dependency.name,
                version=dependency.version,
                packages=packages,
                indent=indent + 2,
                extra=dependency.extra,
                follow_extras=follow_extras,
                default_license=default_license,
            )

    except Exception as e:
        print(
            "  {} [ERROR] Failed to gather {} ({})".format(indent_str, package, str(e))
        )

    return packages[0]


def generate_recipe(package, outdir, python, is_extra=False):
    basename = "{}-{}_{}.bb".format(
        python, package_to_bb_name(package.name), package.version
    )
    bbfile = os.path.join(outdir, basename)

    print("  {}".format(basename))

    if is_extra:
        output = BB_EXTRA_TEMPLATE.format(
            summary=package.summary,
            homepage=package.homepage,
            author=package.author,
            author_email=package.author_email,
            dependencies=" ".join(
                [
                    "{}-{}".format(python, package_to_bb_name(dep.name))
                    for dep in package.dependencies
                ]
            ),
        )
    else:
        output = BB_TEMPLATE.format(
            summary=package.summary,
            md5=package.src_md5,
            sha256=package.src_sha256,
            src_uri=package.src_uri,
            src_dir=package.src_dir,
            license=package.license,
            license_file=package.license_file,
            license_md5=package.license_md5,
            homepage=package.homepage,
            author=package.author,
            author_email=package.author_email,
            dependencies=" ".join(
                [
                    "{}-{}".format(python, package_to_bb_name(dep.name))
                    for dep in package.dependencies
                ]
            ),
            setuptools="3" if python == "python3" else "",
        )

    with open(bbfile, "w") as outfile:
        outfile.write(output)


def parse_requirements(requirements_file, follow_extras=False, default_license=None):
    packages = []

    with open(requirements_file, "r") as infile:
        for package in infile.read().split("\n"):
            package = package.strip()
            if package:
                if not (package.startswith("-e") or package.startswith(".")):
                    parts = [part.strip() for part in package.split("==")]
                    if len(parts) == 2:
                        packages += get_package_info(
                            parts[0],
                            parts[1],
                            follow_extras=follow_extras,
                            default_license=default_license,
                        )
                    elif len(parts) == 1:
                        packages += get_package_info(
                            parts[0],
                            None,
                            follow_extras=follow_extras,
                            default_license=default_license,
                        )
                    else:
                        print("    Unparsed package: {}".format(package))
                else:
                    print("    Skipping: {}".format(package))

    return packages


def write_preferred_versions(packages, outfile, python):
    versions = []
    for package in packages:
        versions.append(
            'PREFERRED_VERSION_{}-{} = "{}"'.format(
                python, package_to_bb_name(package.name), package.version
            )
        )

    with open(outfile, "w") as outfile:
        outfile.write("\n".join(versions))


def generate_recipes(packages, outdir, python, follow_extras=False):
    for package in packages:
        generate_recipe(package, outdir, python)

        if follow_extras:
            extras = [dep for dep in package.dependencies if dep.extra]
            processed = []
            for extra in extras:
                if extra.extra in processed:
                    continue

                processed.append(extra.extra)
                extra_package = package
                extra_package = extra_package._replace(
                    name=package.name + "-{}".format(extra.extra)
                )
                extra_package = extra_package._replace(
                    dependencies=[Dependency(package.name, package.version, None)]
                    + [
                        Dependency(e.name, e.version, None)
                        for e in extras
                        if e.extra == extra.extra
                    ]
                )
                generate_recipe(extra_package, outdir, python, is_extra=True)


def main():
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("--package", "-p", help="The package to process.")
        parser.add_argument(
            "--version", "-v", help="The package version.", default=None
        )
        parser.add_argument("--requirements", "-r", help="The pypi requirements file.")
        parser.add_argument(
            "--extras", "-e", action="store_true", help="Generate recipes for extras."
        )
        parser.add_argument(
            "--outdir", "-o", help="The recipe directory.", default="./"
        )
        parser.add_argument(
            "--python",
            "-y",
            help="The python version to use.",
            default="python",
            choices=["python", "python3"],
        )
        parser.add_argument(
            "--licenses",
            "-l",
            action="store_true",
            help="Output an updated license map upon completion.",
        )
        parser.add_argument(
            "--default-license",
            "-d",
            help="The default license to use when the package license cannot be mapped.",
            default=None,
        )
        args = parser.parse_args()

        print("Gathering info:")
        packages = []
        if args.requirements:
            packages = parse_requirements(
                args.requirements,
                follow_extras=args.extras,
                default_license=args.default_license,
            )
        elif args.package:
            packages = get_package_info(
                args.package,
                args.version,
                follow_extras=args.extras,
                default_license=args.default_license,
            )
        else:
            raise Exception("No packages provided!")

        print("Generating recipes:")
        generate_recipes(packages, args.outdir, args.python, args.extras)

        version_file = os.path.join(args.outdir, "{}-versions.inc".format(args.python))
        write_preferred_versions(packages, version_file, args.python)

        print()
        if args.licenses:
            license_file = os.path.join(args.outdir, "licenses.py")
            with open(license_file, "w") as outfile:
                outfile.write("LICENSES = " + pformat(licenses.LICENSES))

            print("License mappings are available in: {}".format(license_file))

        print("PREFERRED_VERSIONS are available in: {}".format(version_file))

    except Exception as e:
        print(str(e))
        sys.exit(1)
    except KeyboardInterrupt:
        os._exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

from setuptools import setup, find_packages
from os import path
from io import open

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

install_requires = []
with open("requirements.txt", "r") as infile:
    install_requires = [r for r in infile.read().split("\n") if r]

setup(
    name="pipoe",
    version="2019.11",
    description="Generate python bitbake recipes from pypi metadata.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/NFJones/pipoe",
    author="Neil F Jones",
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    keywords="yocto bitbake openembedded",
    packages=["pipoe"],
    python_requires="!=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, <4",
    install_requires=install_requires,
    entry_points={"console_scripts": ["pipoe = pipoe.pipoe:main"]},
)



LICENSES = {
    "AAL": "AAL",
    "AFL-1.2": "AFL-1.2",
    "AFL-2.0": "AFL-2.0",
    "AFL-2.1": "AFL-2.1",
    "AFL-3.0": "AFL-3.0",
    "AGPL-3.0": "AGPL-3.0",
    "ANTLR-PD": "ANTLR-PD",
    "APL-1.0": "APL-1.0",
    "APL2": "Apache-2.0",
    "APSL-1.0": "APSL-1.0",
    "APSL-1.1": "APSL-1.1",
    "APSL-1.2": "APSL-1.2",
    "APSL-2.0": "APSL-2.0",
    "ASL": "Apache-2.0",
    "Adobe": "Adobe",
    "Apache": "Apache-2.0",
    "Apache 2.0": "Apache-2.0",
    "Apache License 2.0": "Apache-2.0",
    "Apache License, Version 2.0": "Apache-2.0",
    "Apache Software License": "Apache-2.0",
    "Apache-1.0": "Apache-1.0",
    "Apache-1.1": "Apache-1.1",
    "Apache-2.0": "Apache-2.0",
    "Artistic-1.0": "Artistic-1.0",
    "Artistic-2.0": "Artistic-2.0",
    "BSD": "BSD",
    "BSD - See ndg/httpsclient/LICENCE file for details": "BSD",
    "BSD 3-Clause": "BSD-3-Clause",
    "BSD 3-Clause License": "BSD-3-Clause",
    "BSD License": "BSD",
    "BSD or Apache License, Version 2.0": "BSD",
    "BSD, Public Domain": "BSD",
    "BSD-2-Clause": "BSD-2-Clause",
    "BSD-3-Clause": "BSD-3-Clause",
    "BSD-4-Clause": "BSD-4-Clause",
    "BSD-derived (http://www.repoze.org/LICENSE.txt)": "BSD",
    "BSD-like": "BSD",
    "BSL-1.0": "BSL-1.0",
    "BitstreamVera": "BitstreamVera",
    "CATOSL-1.1": "CATOSL-1.1",
    "CC-BY-1.0": "CC-BY-1.0",
    "CC-BY-2.0": "CC-BY-2.0",
    "CC-BY-2.5": "CC-BY-2.5",
    "CC-BY-3.0": "CC-BY-3.0",
    "CC-BY-NC-1.0": "CC-BY-NC-1.0",
    "CC-BY-NC-2.0": "CC-BY-NC-2.0",
    "CC-BY-NC-2.5": "CC-BY-NC-2.5",
    "CC-BY-NC-3.0": "CC-BY-NC-3.0",
    "CC-BY-NC-ND-1.0": "CC-BY-NC-ND-1.0",
    "CC-BY-NC-ND-2.0": "CC-BY-NC-ND-2.0",
    "CC-BY-NC-ND-2.5": "CC-BY-NC-ND-2.5",
    "CC-BY-NC-ND-3.0": "CC-BY-NC-ND-3.0",
    "CC-BY-NC-SA-1.0": "CC-BY-NC-SA-1.0",
    "CC-BY-NC-SA-2.0": "CC-BY-NC-SA-2.0",
    "CC-BY-NC-SA-2.5": "CC-BY-NC-SA-2.5",
    "CC-BY-NC-SA-3.0": "CC-BY-NC-SA-3.0",
    "CC-BY-ND-1.0": "CC-BY-ND-1.0",
    "CC-BY-ND-2.0": "CC-BY-ND-2.0",
    "CC-BY-ND-2.5": "CC-BY-ND-2.5",
    "CC-BY-ND-3.0": "CC-BY-ND-3.0",
    "CC-BY-SA-1.0": "CC-BY-SA-1.0",
    "CC-BY-SA-2.0": "CC-BY-SA-2.0",
    "CC-BY-SA-2.5": "CC-BY-SA-2.5",
    "CC-BY-SA-3.0": "CC-BY-SA-3.0",
    "CC0-1.0": "CC0-1.0",
    "CDDL-1.0": "CDDL-1.0",
    "CECILL-1.0": "CECILL-1.0",
    "CECILL-2.0": "CECILL-2.0",
    "CECILL-B": "CECILL-B",
    "CECILL-C": "CECILL-C",
    "CPAL-1.0": "CPAL-1.0",
    "CPL-1.0": "CPL-1.0",
    "CUA-OPL-1.0": "CUA-OPL-1.0",
    "ClArtistic": "ClArtistic",
    "DSSSL": "DSSSL",
    "Dual License": "BSD",
    "ECL-1.0": "ECL-1.0",
    "ECL-2.0": "ECL-2.0",
    "EDL-1.0": "EDL-1.0",
    "EFL-1.0": "EFL-1.0",
    "EFL-2.0": "EFL-2.0",
    "EPL-1.0": "EPL-1.0",
    "EPL-2.0": "EPL-2.0",
    "EUDatagrid": "EUDatagrid",
    "EUPL-1.0": "EUPL-1.0",
    "EUPL-1.1": "EUPL-1.1",
    "Elfutils-Exception": "Elfutils-Exception",
    "Entessa": "Entessa",
    "ErlPL-1.1": "ErlPL-1.1",
    "Expat license": "MIT",
    "Expat (MIT/X11)": "MIT",
    "Fair": "Fair",
    "Frameworx-1.0": "Frameworx-1.0",
    "FreeType": "FreeType",
    "GFDL-1.1": "GFDL-1.1",
    "GFDL-1.2": "GFDL-1.2",
    "GFDL-1.3": "GFDL-1.3",
    "GNU GPLv3+": "GPL-3.0",
    "GNU General Public License Version 3": "GPL-3.0",
    "GNU LGPL": "LGPL-2.0",
    "GPL": "GPL-1.0",
    "GPL V2 or later": "GPL-2.0",
    "GPL-1.0": "GPL-1.0",
    "GPL-2-with-bison-exception": "GPL-2-with-bison-exception",
    "GPL-2.0": "GPL-2.0",
    "GPL-2.0-with-GCC-exception": "GPL-2.0-with-GCC-exception",
    "GPL-2.0-with-autoconf-exception": "GPL-2.0-with-autoconf-exception",
    "GPL-2.0-with-classpath-exception": "GPL-2.0-with-classpath-exception",
    "GPL-2.0-with-font-exception": "GPL-2.0-with-font-exception",
    "GPL-3.0": "GPL-3.0",
    "GPL-3.0-with-GCC-exception": "GPL-3.0-with-GCC-exception",
    "GPL-3.0-with-autoconf-exception": "GPL-3.0-with-autoconf-exception",
    "GPLv3+": "GPLv3",
    "HPND": "HPND",
    "IPA": "IPA",
    "IPL-1.0": "IPL-1.0",
    "ISC": "ISC",
    "ISC license": "ISC",
    "LGPL": "LGPL-2.0",
    "LGPL-2.0": "LGPL-2.0",
    "LGPL-2.1": "LGPL-2.1",
    "LGPL-3.0": "LGPL-3.0",
    "LGPLv2": "LGPL-2.0",
    "LGPLv2+": "LGPL-2.0",
    "LGPLv3": "LGPL-3.0",
    "LPGL, see LICENSE file.": "LGPL-1.0",
    "LPL-1.02": "LPL-1.02",
    "LPPL-1.0": "LPPL-1.0",
    "LPPL-1.1": "LPPL-1.1",
    "LPPL-1.2": "LPPL-1.2",
    "LPPL-1.3c": "LPPL-1.3c",
    "Libpng": "Libpng",
    "License :: OSI Approved :: Apache Software License": "Apache-2.0",
    "License :: OSI Approved :: MIT License (http://opensource.org/licenses/MIT)": "MIT",
    "MIT": "MIT",
    "MIT License": "MIT",
    "MIT license": "MIT",
    "MIT/X11": "MIT",
    "MPL v2": "MPL-2.0",
    "MPL-1.0": "MPL-1.0",
    "MPL-1.1": "MPL-1.1",
    "MPL-2.0": "MPL-2.0",
    "MPLv2.0, MIT Licences": "MIT",
    "MS-PL": "MS-PL",
    "MS-RL": "MS-RL",
    "MirOS": "MirOS",
    "Modified BSD License": "BSD",
    "Motosoto": "Motosoto",
    "Multics": "Multics",
    "NASA-1.3": "NASA-1.3",
    "NCSA": "NCSA",
    "NGPL": "NGPL",
    "NPOSL-3.0": "NPOSL-3.0",
    "NTP": "NTP",
    "Nauman": "Nauman",
    "New BSD": "BSD",
    "Nokia": "Nokia",
    "OASIS": "OASIS",
    "OCLC-2.0": "OCLC-2.0",
    "ODbL-1.0": "ODbL-1.0",
    "OFL-1.1": "OFL-1.1",
    "OGTSL": "OGTSL",
    "OLDAP-2.8": "OLDAP-2.8",
    "OSL-1.0": "OSL-1.0",
    "OSL-2.0": "OSL-2.0",
    "OSL-3.0": "OSL-3.0",
    "OpenSSL": "OpenSSL",
    "PD": "PD",
    "PHP-3.0": "PHP-3.0",
    "PSF": "Python-2.0",
    "PSF License": "Python-2.0",
    "PSF license": "Python-2.0",
    "PSFL": "Python-2.0",
    "PostgreSQL": "PostgreSQL",
    "Proprietary": "Proprietary",
    "Public Domain": "PD",
    "Public domain": "PD",
    "Python Software Foundation License": "Python-2.0",
    "Python style": "Python-2.0",
    "Python-2.0": "Python-2.0",
    "QPL-1.0": "QPL-1.0",
    "RHeCos-1": "RHeCos-1",
    "RHeCos-1.1": "RHeCos-1.1",
    "RPL-1.5": "RPL-1.5",
    "RPSL-1.0": "RPSL-1.0",
    "RSCPL": "RSCPL",
    "Ruby": "Ruby",
    "SAX-PD": "SAX-PD",
    "SGI-1": "SGI-1",
    "SPL-1.0": "SPL-1.0",
    "Simple-2.0": "Simple-2.0",
    "Sleepycat": "Sleepycat",
    "SugarCRM-1": "SugarCRM-1",
    "SugarCRM-1.1.3": "SugarCRM-1.1.3",
    "This software released into the public domain. Anyone is free to copy,": "PD",
    "Two-clause BSD license": "BSD-2-Clause",
    "UCB": "UCB",
    "UNKNOWN": "CLOSED",
    "VSL-1.0": "VSL-1.0",
    "W3C": "W3C",
    "WXwindows": "WXwindows",
    "Watcom-1.0": "Watcom-1.0",
    "XFree86-1.0": "XFree86-1.0",
    "XFree86-1.1": "XFree86-1.1",
    "XSL": "XSL",
    "Xnet": "Xnet",
    "YPL-1.1": "YPL-1.1",
    "ZPL 2.1": "ZPL-2.0",
    "ZPL-1.1": "ZPL-1.1",
    "ZPL-2.0": "ZPL-2.0",
    "ZPL-2.1": "ZPL-2.1",
    "Zimbra-1.3": "Zimbra-1.3",
    "Zlib": "Zlib",
    "eCos-2.0": "eCos-2.0",
    "gSOAP-1": "gSOAP-1",
    "gSOAP-1.3b": "gSOAP-1.3b",
    "http://creativecommons.org/publicdomain/zero/1.0/": "PD",
    "http://opensource.org/licenses/MIT": "MIT",
    "public domain, Python, 2-Clause BSD, GPL 3 (see COPYING.txt)": "PD",
}

